// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"github.com/jinzhu/gorm"
	"time"
	"fmt"
)

const (
	_ = iota
	LogTypeCompany
	LogTypeProject
	LogTypeProvider
	LogTypeGovernment
)

type TLog struct {
	ID int64 `gorm:"column:id" json:"id" form:"id"`
	Uid int64 `gorm:"column:uid" json:"uid" form:"uid"`
	Name string `gorm:"column:name" json:"name" form:"name"`
	Operation string `gorm:"column:operation" json:"operation" form:"operation"`
	Detail string `gorm:"column:detail" json:"detail" form:"detail"`
	Result int `gorm:"column:result" json:"result" form:"result"`
	CreatedAt time.Time `gorm:"column:created_at" json:"created_at" form:"created_at"`
	Type int32 `gorm:"-"`
	ProjectId int64
}

func (p TLog) TableName() string {
	switch p.Type {
	case LogTypeCompany:
		return "t_company_log"
	case LogTypeProject:
		return "t_project_log"
	case LogTypeGovernment:
		return "t_system_log"
	case LogTypeProvider:
		return "t_provider_log"

	}
	return ""
}

func (p *TLog) Insert(db *gorm.DB) error {
	return db.Create(p).Error
}


func (p *TLog) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *TLog) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).First(p).Error
}

func (p *TLog) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}


func (p *TLog) Save(db *gorm.DB) error {
	return db.Save(p).Error
}

func (p *TLog) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	return ret, err
}

func (p *TLog) List(db *gorm.DB, where map[string]interface{}, page int32) (list []TLog, err error) {
	offset := (page - 1) * int32(PageSize)
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		result := db.Table(p.TableName()).Where(cond, val...).Limit(PageSize).Offset(offset).Find(&list)

		return list, result.Error
	}
	result := db.Table(p.TableName()).Limit(10).Offset(offset).Find(&list)

	return list, result.Error
}

type LogInfo struct {
	Name                 string
	Operation            string
	Result               int32
	Detail               string
	Uid                  int64
	CreatedAt             time.Time
	ProjectName          string
	ProjectId            int64
	ProjectNo            string
	SafetyRecordNo       string
}

func logListWithProjectSql(page int32, start, end int64, tab string)(string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.name, t1.operation, t1.uid, t1.created_at, t1.project_id, t1.detail, t1.result,"+
		"t2.project_no, t2.name as project_name, t2.safety_record_no from db_smart_log.%s as t1 left join db_smart.t_project as t2 on t1.project_id=t2.id ", tab)
	countSql := fmt.Sprintf("select count(1) as count from db_smart_log.%s as t1 left join db_smart.t_project as t2 on t1.project_id=t2.id ", tab)
	args := []interface{}{}
	whereArray := []string{}
	if start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.created_at>='%s'", time.Unix(start, 0).Format("2006-01-02")))
	}
	if end > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.created_at<'%s'", time.Unix(end, 0).Format("2006-01-02")))
	}

	where := ""

	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf(" where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}
	offset := (page - 1) *int32(page)

		sql = fmt.Sprintf("%s %s limit %d offset %d", sql, where, PageSize, offset)
		countSql = fmt.Sprintf("%s %s", countSql, where)

	return sql, countSql, args
}

func (p *TLog) ListWithProject(db *gorm.DB, page int32, start, end int64) ([]LogInfo, int64, error) {
	type ResultCount struct {
		Count int64
	}
	array := []ResultCount{}
	ret := []LogInfo{}
	var err error

	sql, countSql, args := logListWithProjectSql(page, start, end, p.TableName())

	err = db.Raw(countSql, args...).Scan(&array).Error
	if err != nil {
		return nil, 0, err
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}

	err = db.Raw(sql, args...).Scan(&ret).Error
	if err != nil {
		return nil, array[0].Count, err
	}
	return ret, array[0].Count, nil
}



